class ModelRepository:
    def __init__(self, url, branch=None, access_token=None):
        self.url = url
        self.branch = branch if branch is not None else "main"
        self.access_token = access_token

    def __repr__(self):
        res = f"{self.__module__}.{self.__class__.__name__}("
        res += f"\n      url={self.url}"
        res += f"\n      branch={self.branch}"
        res += "\n    )"
        return res


class ModelParams:
    def __init__(self, url, config_url="", upload=False, credentials=None):
        self.url = url
        self.config_url = config_url
        self.upload = upload
        self.credentials = credentials

    def __repr__(self):
        res = f"{self.__module__}.{self.__class__.__name__}("
        res += f"\n      url={self.url}"
        res += f"\n    )"
        return res


class Model:
    @staticmethod
    def create(project, name, repository, params):
        if name is None or repository is None or params is None:
            return None

        repository = ModelRepository(**repository)
        params = ModelParams(**params)

        session = project._session
        response = session._put(
            f"api/model/undefined/{project.id}",
            json={
                "name": name,
                "repository_url": repository.url,
                "branch": repository.branch,
                "access_token": repository.access_token,
                "model_url": params.url,
                "model_config_url": params.config_url,
                "upload_params": params.upload,
                "projectId": project.id,
            },
        )
        model_id = response["id"]

        if params.upload:
            session._upload(params.url, f"api/model/{model_id}/upload")
            if params.config_url:
                session._upload(params.config_url, f"api/model/{model_id}/upload")

        return Model(project, model_id, name, repository, params)

    def __init__(self, project, id, name, repository, params):
        self.project = project
        self.id = id
        self.name = name
        self.repository = repository
        self.params = params

    def __repr__(self):
        res = f"{self.__module__}.{self.__class__.__name__}("
        res += f"\n  id={self.id}"
        res += f"\n  name={self.name}"
        res += f"\n  repository={self.repository}"
        res += f"\n  params={self.params}"
        res += f"\n)"
        return res

    def delete(self):
        self.project._session._delete(f"api/model/{self.id}/{self.project.id}")



from datetime import datetime
from re import T
from pathlib import Path
import click
import efemarai as ef
from rich.console import Console
from rich.table import Table


console = Console()


@click.group()
def main():
    """Efemarai CLI."""
    pass


@main.command()
@click.option("-c", "--config", help="Optional configuration file.")
def init(config):
    """Initialize Efemarai."""
    ef.Session()._user_setup(config_file=config)


@main.group()
def project():
    """Manage projects."""
    pass


@project.command("create")
@click.argument("config", required=True)
@click.option(
    "--exists_ok", default=False, help="Append differences if project already exists."
)
def project_create(config, exists_ok):
    """Create a project following the specified configuration file.

    CONFIG - configuration file."""
    result = ef.Session().load(config, exists_ok=exists_ok)
    print(result)


@project.command("list")
def project_list():
    """Lists the projects associated with the current user."""
    table = Table(box=None)
    [table.add_column(x) for x in ["Id", "Name", "Problem Type"]]
    for m in ef.Session().projects:
        table.add_row(m.id, m.name, str(m.problem_type))
    console.print(table)


@main.group()
def model():
    """Manage models."""
    pass


@model.command("list")
def model_list():
    """Lists the models in the current project."""
    project = _get_project()
    if not project:
        return

    table = Table(box=None)
    [table.add_column(x) for x in ["Id", "Name"]]
    for m in project.models:
        table.add_row(m.id, m.name)
    console.print(table)


@model.command("create")
@click.argument("config", required=True)
def model_create(config):
    """Create a model in the current project.

    CONFIG - configuration file."""
    project = _get_project()
    if not project:
        return
    config_file = ef.Session._read_config(config)
    model = project.create_model(**config_file["model"])
    return model


@model.command("delete")
@click.argument("model", required=True)
def model_delete(model):
    """Delete a model from the current project.

    MODEL - the name or ID of the model."""
    project = _get_project()
    if not project:
        return

    if _check_for_multiple_entities(project.models, model):
        console.print("There are multiple models with the given:\n")
        models = [t for t in project.models if t.name == test]
        _print_table(models)
        console.print(
            f"\nRun the command with a specific model id: [bold green]$ efemarai model delete {models[0].id}",
        )
        return

    model = project.model(model)
    model.delete()


@main.group()
def domain():
    """Manage domains."""
    pass


@domain.command("list")
def domain_list():
    """Lists the domains in the current project."""
    project = _get_project()
    if not project:
        return

    table = Table(box=None)
    [table.add_column(x) for x in ["Id", "Name"]]
    for d in project.domains:
        table.add_row(d.id, d.name)
    console.print(table)


@domain.command("create")
@click.argument("config", required=True)
def domain_create(config):
    """Create a domain in the current project.

    CONFIG - configuration file."""
    project = _get_project()
    if not project:
        return
    config_file = ef.Session._read_config(config)
    domain = project.create_domain(**config_file["domain"])
    return domain


@domain.command("delete")
@click.argument("domain", required=True)
def domain_delete(domain):
    """Delete a domain from the current project.

    DOMAIN - the name or ID of the domain."""
    project = _get_project()
    if not project:
        return

    if _check_for_multiple_entities(project.domains, domain):
        console.print("There are multiple domains with the given name:\n")
        domains = [t for t in project.domains if t.name == test]
        _print_table(domains)
        console.print(
            f"\nRun the command with a specific domain id: [bold green]$ efemarai domain delete {domains[0].id}",
        )
        return

    domain = project.domain(domain)
    domain.delete()


@domain.command("download")
@click.argument("domain", required=True)
@click.option("-o", "--output", default=None, help="Optional domain output file.")
def domain_download(domain, output):
    """Download a domain.

    DOMAIN - the name of the domain."""
    project = _get_project()
    if not project:
        return

    domain = project.domain(domain)
    filename = domain.download(filename=output)
    console.print(
        (f":heavy_check_mark: Downloaded '{domain.name}' reports to: \n  {filename}"),
        style="green",
    )


@main.group()
def dataset():
    """Manage datasets."""
    pass


@dataset.command("list")
def dataset_list():
    """Lists the datasets in the current project."""
    project = _get_project()
    if not project:
        return

    table = Table(box=None)
    [table.add_column(x) for x in ["Id", "Name", "Loaded"]]
    for d in project.datasets:
        table.add_row(d.id, d.name, str(d.loaded))
    console.print(table)


@dataset.command("create")
@click.argument("config", required=True)
def dataset_create(config):
    """Create a dataset in the current project.

    CONFIG - configuration file."""
    project = _get_project()
    if not project:
        return
    config_file = ef.Session._read_config(config)
    dataset = project.create_dataset(**config_file["dataset"])
    return dataset


@dataset.command("delete")
@click.argument("dataset", required=True)
def dataset_delete(dataset):
    """Delete a dataset from the current project.

    DATASET - the name or ID of the dataset."""
    project = _get_project()
    if not project:
        return

    if _check_for_multiple_entities(project.datasets, dataset):
        console.print("There are multiple datasets with the given name:\n")
        datasets = [t for t in project.datasets if t.name == test]
        _print_table(datasets)
        console.print(
            f"\nRun the command with a specific dataset id: [bold green]$ efemarai dataset delete {datasets[0].id}",
        )
        return

    dataset = project.dataset(dataset)
    dataset.delete()


@main.group()
def test():
    """Manage stress tests."""
    pass


@test.command("list")
def test_list():
    """Lists the stress tests in the current project."""
    project = _get_project()
    if not project:
        return

    _print_table(project.stress_tests)


@test.command("run")
@click.argument("config", required=True)
def test_run(config):
    """Run a stress test.

    CONFIG - stress test configuration file."""
    project = _get_project()
    if not project:
        return

    config_file = ef.Session._read_config(config)
    test = project.create_stress_test(**config_file["test"])
    cfg = ef.Session._read_config()
    console.print(f"{cfg['url']}project/{project.id}/runs/{test.id}")


@test.command("delete")
@click.argument("test", required=True)
def test_delete(test):
    """Delete a stress test from the current project.

    TEST - the name or ID of the stress test."""
    project = _get_project()
    if not project:
        return

    if _check_for_multiple_entities(project.stress_tests, test):
        console.print("There are multiple stress tests with the given name:\n")
        tests = [t for t in project.stress_tests if t.name == test]
        _print_table(tests)
        console.print(
            f"\nRun the command with a specific stress test id: [bold green]$ efemarai test delete {tests[0].id}",
        )
        return

    test = project.stress_test(test)
    test.delete()


@test.command("download")
@click.argument("test", required=True)
@click.option("--min_score", default=0, help="Minimum score for the samples.")
@click.option("--include_dataset", default=False, help="Include original test dataset.")
@click.option("--path", default=None, help="Path to the downloaded files.")
@click.option("--unzip", default=True, help="Whether to unzip the resulting file.")
@click.option("--ignore_cache", default=False, help="Ignore local cache.")
def test_download(test, min_score, include_dataset, path, unzip, ignore_cache):
    """Download the stress test vulnerabilities dataset.

    TEST - the name or ID of the stress test to download."""
    project = _get_project()
    if not project:
        return

    test = project.stress_test(test)
    test.vulnerabilities_dataset(
        min_score=min_score,
        include_dataset=include_dataset,
        path=path,
        unzip=unzip,
        ignore_cache=ignore_cache,
    )


@test.command("reports")
@click.argument("test", required=True)
@click.option("-o", "--output", default=None, help="Optional output file.")
def test_reports(test, output):
    """Export the stress test reports.

    TEST - name or ID of the stress test."""
    project = _get_project()
    if not project:
        return

    test = project.stress_test(test)
    filename = test.download_reports(filename=output)
    console.print(
        (f":heavy_check_mark: Downloaded '{test.name}' reports to: \n  {filename}"),
        style="green",
    )


def _print_table(tests):
    table = Table(box=None)
    [table.add_column(x) for x in ["Id", "Name", "Model", "Dataset", "Domain", "State"]]
    for t in tests:
        table.add_row(
            t.id, t.name, t.model.name, t.dataset.name, t.domain.name, str(t.state)
        )
    console.print(table)


def _check_for_multiple_entities(entities, name):
    length = len(list(filter(lambda x: x.name == name, entities)))
    return length > 1


def _get_project():
    if not Path("efemarai.yaml").is_file():
        console.print(
            f":poop: Cannot find 'efemarai.yaml' in the current directory.", style="red"
        )
        exit(1)

    ef_file = ef.Session()._load_config_file("efemarai.yaml")
    if "project" in ef_file and "name" in ef_file["project"]:
        name = ef_file["project"]["name"]
        project = ef.Session().project(name)
        if not project:
            console.print(f"Project '{name}' is not existing.")
            return

        return project

    console.print(
        f":poop: 'efemarai.yaml' file not configured properly (does not container project and name within).",
        style="red",
    )
    exit(1)


if __name__ == "__main__":
    main()



from efemarai.dataset import Dataset
from efemarai.domain import Domain
from efemarai.model import Model, ModelParams, ModelRepository
from efemarai.problem_type import ProblemType
from efemarai.stress_test import StressTest


class Project:
    """Provides project related functionality."""

    @staticmethod
    def create(session, name, description, problem_type, exists_ok=False):
        if name is None or not ProblemType.has(problem_type):
            return None

        existing_project = next((p for p in session.projects if p.name == name), None)
        if existing_project:
            if exists_ok:
                return existing_project
            else:
                raise ValueError(f"Project {name} already exists and exists_ok=False.")

        response = session._put(
            "api/project",
            json={
                "name": name,
                "description": description,
                "problem_type": problem_type,
            },
        )
        return Project(session, response["id"], name, description, problem_type)

    def __init__(self, session, id, name, description, problem_type):
        self._session = session
        self.id = id
        self.name = name
        self.description = description
        self.problem_type = ProblemType(problem_type)

    def __repr__(self):
        res = f"{self.__module__}.{self.__class__.__name__}("
        res += f"\n  id={self.id}"
        res += f"\n  name={self.name}"
        res += f"\n  description={self.description}"
        res += f"\n  problem_type={self.problem_type}"
        res += f"\n)"
        return res

    @property
    def models(self):
        """Returns a list of the models associated with the project."""

        return [
            Model(
                self,
                model["id"],
                model["name"],
                repository=ModelRepository(
                    url=model["repository_url"],
                    branch=model["branch"],
                    access_token=model["access_token"],
                ),
                params=ModelParams(url=model["model_url"]),
            )
            for model in self._session._get(f"api/models/{self.id}")
        ]

    def model(self, model):
        """Returns the model specified by the name."""

        model = next((m for m in self.models if m.name == model or m.id == model), None)
        return model

    def create_model(
        self, name, repository=None, params=None, exists_ok=False, **kwargs
    ):
        """Creates a model."""

        existing_model = next((m for m in self.models if m.name == name), None)
        if existing_model:
            if exists_ok:
                return existing_model
            else:
                raise ValueError(f"Model {name} already exists and exists_ok=False.")

        model = Model.create(self, name, repository, params)
        return model

    @property
    def datasets(self):
        """Returns a list of the datasets associated with the project."""

        return [
            Dataset(
                self,
                dataset["id"],
                dataset["name"],
                dataset["format"],
                dataset["stage"],
                dataset["data_url"],
                dataset["annotations_url"],
                dataset["loaded"],
                Dataset._parse_classes(dataset["classes"]),
            )
            for dataset in self._session._get(f"api/datasets/{self.id}")
        ]

    def dataset(self, dataset):
        """Returns the dataset specified by the name or id."""

        dataset = next(
            (d for d in self.datasets if d.name == dataset or d.id == dataset), None
        )
        return dataset

    def create_dataset(
        self,
        name,
        format=None,
        stage=None,
        data_url=None,
        annotations_url=None,
        credentials=None,
        upload=False,
        num_datapoints=None,
        mask_generation=None,
        exists_ok=False,
        **kwargs,
    ):
        """Creates a dataset."""

        existing_dataset = next((d for d in self.datasets if d.name == name), None)
        if existing_dataset:
            if exists_ok:
                return existing_dataset
            else:
                raise ValueError(f"Dataset {name} already exists and exists_ok=False.")

        dataset = Dataset.create(
            self,
            name,
            format,
            stage,
            data_url,
            annotations_url,
            credentials,
            upload,
            num_datapoints,
            mask_generation,
        )

        return dataset

    @property
    def domains(self):
        """Returns a list of the domains associated with the project."""

        return [
            Domain(
                self,
                domain["id"],
                domain["name"],
                domain["transformations"],
                domain["graph"],
            )
            for domain in self._session._get(f"api/domains/{self.id}")
        ]

    def domain(self, domain):
        """Returns the domain specified by the name or id."""

        domain = next(
            (d for d in self.domains if d.name == domain or d.id == domain), None
        )
        return domain

    def create_domain(
        self, name, transformations=None, graph=None, exists_ok=False, **kwargs
    ):
        """Creates a domain."""

        existing_domain = next((d for d in self.domains if d.name == name), None)
        if existing_domain:
            if exists_ok:
                return existing_domain
            else:
                raise ValueError(f"Domain {name} already exists and exists_ok=False.")

        domain = Domain.create(self, name, transformations, graph)
        return domain

    @property
    def stress_tests(self):
        """Returns a list of the stress tests associated with the project."""

        return [
            StressTest(
                self,
                test["id"],
                test["name"],
                test["model"]["id"],
                test["domain"]["id"],
                test["dataset"]["id"],
                test["states"][-1]["name"],
                test["reports"],
            )
            for test in self._session._get(f"api/getRuns/{self.id}")["objects"]
        ]

    def stress_test(self, test):
        """Returns the stress test specified by the name or id."""

        test = next(
            (t for t in self.stress_tests if t.name == test or t.id == test), None
        )
        return test

    def create_stress_test(
        self,
        name,
        model=None,
        domain=None,
        dataset=None,
        num_samples=None,
        num_runs=None,
        concurrent_runs=None,
        **kwargs,
    ):
        """Creates a stress test."""

        test = StressTest.create(
            self,
            name,
            model,
            domain,
            dataset,
            num_samples,
            num_runs,
            concurrent_runs,
        )

        return test

    def delete(self):
        """Deletes a project, including the domains, datasets and models."""

        for domain in self.domains:
            domain.delete()

        for dataset in self.datasets:
            dataset.delete()

        for model in self.models:
            model.delete()

        self._session._delete(f"api/project/{self.id}")



from enum import Enum

class ProblemType(Enum):
    Classification = "Classification"
    ObjectDetection = "ObjectDetection"
    SemanticSegmentation = "SemanticSegmentation"
    InstanceSegmentation = "InstanceSegmentation"
    ImageRegression = "ImageRegression"
    Keypoints = "Keypoints"

    @classmethod
    def has(cls, value):
        try:
            cls(value)
            return True
        except ValueError:
            return False



import re
import os

from setuptools import setup


def get_version():
    filename = os.path.join(
        os.path.abspath(os.path.dirname(__file__)), "efemarai/__init__.py"
    )
    with open(filename, "r") as f:
        for line in f.read().splitlines():
            match = re.search("^__version__ = [\"'](?P<version>.*)[\"']$", line)
            if match:
                return match.group("version")

        raise RuntimeError("Unable to find version string.")


def get_requirements():
    with open("requirements.txt") as f:
        requirements = [
            line
            for line in f.read().splitlines()
            if not line.startswith("#") and len(line) > 0
        ]
    return requirements


def get_readme():
    with open("README.md") as f:
        readme = f.read()
    return readme


setup(
    name="efemarai",
    version=get_version(),
    description="A CLI and SDK for interacting with the Efemarai ML testing platform.",
    long_description=get_readme(),
    long_description_content_type="text/markdown",
    author="Efemarai",
    author_email="support@efemarai.com",
    url="https://www.efemarai.com/",
    license="MIT license",
    packages=["efemarai"],
    entry_points={"console_scripts": ["efemarai = efemarai.cli:main"]},
    install_requires=get_requirements(),
    python_requires=">=3.6",
    zip_safe=False,
)
